function findbrightest2, linex, linestrength, nfind, xrange=xrange, nsegments=nsegments, minrelstrength=minrelstrength, indices=indices

   ; find the "brightest" nfind lines
   ; new algorithm that does a rolling scan .
   ; testing seems to indicate that this is a superior algorithm.

   if nfind ge n_elements(linex) then return, linex
   if n_elements(nsegments) eq 0 then nsegments = 4
   if n_elements(minrelstrength) eq 0 then minrelstrength = 0.

   if n_elements(xrange) gt 0 then xmin = xrange[0] else xmin = min(linex)-0.1
   if n_elements(xrange) gt 1 then xmax = xrange[1] else xmax = max(linex)+0.1
   minstrength = minrelstrength*max(linestrength)

   w = where(linex ge xmin and linex le xmax and linestrength ge minstrength)
   x = linex[w]   
   y = linestrength[w]
 
   n = n_elements(x)

   segsize = (xmax-xmin)/nsegments
   segrad = segsize/2.
   step = segrad/2.
   nstep = round((xmax-xmin)/step)
   nperstep = ceil(4*nfind*step/(xmax-xmin)) ; oversample by factor 4
   collectlinex = [-1]
   ; this actually risks underweighting very bright lines close to the edge- 
   ; if the oversampling rate is high they have fewer chances
   for c = xmin, xmax+(0.1*step), step do begin
       ; print, clip(c-segrad)+'-'+clip(c)+'-'+clip(c+segrad)+':', format='(A,$)'
       wc = where(x ge c-segrad and x lt c+segrad, ct)
       if ct eq 0 then continue
       ord = reverse(sort(y[wc]))
       addlines = x[wc[ord[0:(ct-1)<(nperstep-1)]]]
       ; print, addlines
       collectlinex = [collectlinex, addlines]
   endfor
   collectlinex = collectlinex[1:*]
   collectlinex = collectlinex[sort(collectlinex)]
   ulinesx = unique(collectlinex, count=ulinesxn)
   ulinesy = fltarr(n_elements(ulinesx))
   ; print, float(ulinesx)
   for i = 0, n_elements(ulinesy)-1 do ulinesy[i] = y[(where(x eq ulinesx[i])) [0]]
   boost =  (1.0*ulinesy-min(ulinesy))/(max(ulinesy)-1.0*min(ulinesy))  
   ; print, float(ulinesxn)
   ; print, float(ulinesy)
   ; print, boost
   importance = ulinesxn+boost ; tiebreaker
   ; print, importance
   ord = reverse(sort(importance))
   ulinesx = ulinesx

   xout = ulinesx[ord[0:(nfind-1)<(n_elements(ord)-1)]]
   ord = sort(xout)
   xout = xout[ord]
   indices = intarr(n_elements(xout))
   for i = 0, n_elements(xout)-1 do indices[i] = (where(linex eq xout[i])) [0]
   ; yout = yout[ord]

   return, xout

end


pro testfindbrightest2, seed=seed
   if n_elements(seed) eq 0 then dum = randomu(seed)
   seed = seed[0]
   print, 'seed = ',seed

   x = 3+indgen(96)
   y = 10.0^(1.95*randomu(seed,100))
   window, 0

   plot, x, y, psym=1
   brightestx5 = findbrightest2(x,y,5,indices=w5)
   brightestx10 = findbrightest2(x,y,10,indices=w10)
   brightestold = findbrightest(x,y,10,indices=wold) 

   print, x[w5]
   print, fix(round(y[w5]))

   oplot, x[wold], y[wold], psym=6, color=rgb(0,155,0), symsize=1.5
   oplot, x[w10], y[w10], psym=2, color=rgb(255,255,0)
   oplot, x[w5], y[w5], psym=2, color=rgb(255,0,0)
end

