function findtrace, image, xsearch=xsearch, ysearch=ysearch, apmin=apmin, apmax=apmax, sigma=sigma, minsigma=minsigma, peak=peak

      if size(image, /tn) eq 'STRING' then img = readfits(image) else img = image

      s = size(img)
      nx = s[1]
      ny = s[2]

      ; blind search algorithm not rigorously tested for faint objects

      if n_elements(ysearch) ge 2 then begin
         ysearchmin = round(ysearch[0]) > (0)
         ysearchmax = round(ysearch[1]) < (ny-1)
      endif
      if n_elements(ysearch) eq 0 then begin
         ysearchmin = 0  + 2
         ysearchmax = ny - 2
      endif

      if n_elements(xsearch) ge 2 then begin
         xmin = round(xsearch[0])
         xmax = round(xsearch[1])
      endif else begin
         xmin = (nx*4)/10
         xmax = (nx*6)/10
      endelse

      print, '  Searching for trace between y=',clip(ysearchmin),'-',clip(ysearchmax), '  (x=', clip(xmin),'-',clip(xmax),')'

      wavsum = fltarr(ysearchmax-ysearchmin)
      for y = ysearchmin, ysearchmax-1 do begin
         wavsum[y-ysearchmin] = median(img[xmin:xmax,y]) 
      endfor
      kernel = [0.2,1.0,1.5,1.0,0.2]
      if n_elements(kernel) gt n_elements(wavsum) then kernel = [0.3,1.0,0.3]
      if n_elements(kernel) gt n_elements(wavsum) then kernel = [1.0]
      kernel = kernel / total(kernel)
      wavsumconv = convol(wavsum, kernel, /edge_truncate)

      centersigma = max(wavsum)/stddev(wavsum, /nan)   ; stdev is a huge overestimate for a bright star being present.
      if n_elements(minsigma) gt 0 then if centersigma lt minsigma then return, !values.f_nan
      if centersigma lt 2 then begin
         print, '  WARNING:  Cannot find the source!   Best center is only ', clip(fpr(centersigma,5.2)), '-sigma above background.'
         print, '  Continuing anyway, but you will likely want to rerun after specifying ysearch.'
      endif

      avcenty = ysearchmin + (where(wavsumconv eq max(wavsumconv))) [0] ; max val after convolution
      if avcenty lt ysearchmin then avcenty = ysearchmin
      if avcenty gt ysearchmax then avcenty = ysearchmax

      profmax = max(wavsum)
      baseline = median(wavsum, /even)
      for y = avcenty, ysearchmax-1 do begin
         if wavsum[y-ysearchmin] lt baseline + (profmax-baseline)*0.2 then begin
            apmax = y
            break
         endif
      endfor
      for y = avcenty, ysearchmin, -1 do begin  ; formerly ysearchmin-1, which caused a rare error...
         if wavsum[y-ysearchmin] lt baseline + (profmax-baseline)*0.2 then begin
            apmin = y
            break
         endif
      endfor
     
      if n_elements(apmax) eq 0 then apmax = ysearchmax
      if n_elements(apmin) eq 0 then apmin = ysearchmin

      if (apmax-avcenty) gt 6 then apmax = avcenty + 5
      if (avcenty-apmin) gt 6 then apmin = avcenty - 5

      sigma = centersigma
      peak = profmax
 
      print, '  Found object at y=', strtrim(avcenty,2), ' (+', clip(apmax-avcenty), ' -', clip(avcenty-apmin), ')'

   return, avcenty

end

