pro flatcombine, filelist, outfile, removeobjects=removeobjects, response=response, indresp=indresp, yresp=yresp, lines=lines, objthresh=objthresh, objbuffer=objbuffer, verbose=verbose, mediani=mediani, meani=meani, trimlo=trimlo, trimhi=trimhi, mincounts=mincounts, maxcounts=maxcounts, type=type, normzonein=normzonein, linewidth=linewidth, lineblocks=lineblocks, satlevel=satlevel, satradius=satradius, xnormkey=xnormkey, slitnorm=slitnorm, xdiscontinuity=xdiscontinuity


; combine a bunch of flats with optional response division

; Now can accept flats of different sizes and align by physical coordinates.  Use with caution.

algorithm = 'median' ; default
if keyword_set(mediani) then algorithm = 'median'
if keyword_set(meani)   then algorithm = 'mean'  ; the 'i' is because otherwise it overwrites mean()?
if n_elements(trimlo) eq 0 then trimlo = (n_elements(filelist)+1)/4
if n_elements(trimhi) eq 0 then trimhi = trimlo
if n_elements(mincounts) eq 0 then mincounts = 100
if n_elements(maxcounts) eq 0 then maxcounts = 55000
if n_elements(satradius) eq 0 then satradius = 50

if n_elements(outfile) eq 0 then begin
  filearr = strsplit(filelist[0],'.', /extract)
  fileroot = filearr[0]
  fileroot = repstr(fileroot,'*','')
  fileroot = repstr(fileroot,'?','')
  if strpos(fileroot,'flat') eq -1 then  outfile = fileroot + '.flat.fits' else $
                                         outfile = fileroot + '.fits'  ; could overwrite an input file...
endif


if n_elements(filelist) eq 1 then begin
  if n_elements(file_search(filelist)) eq 0 then begin
     print, 'No matching file(s): ', filelist
  endif
  if n_elements(file_search(filelist)) eq 1 then begin
     files = grabcolumn(filelist,0,/str)
     files = files[where(files ne '')]
  endif
  if n_elements(files) eq 1 and n_elements(filelist) eq 1 then files = file_search(filelist)
endif else begin
  files = filelist
endelse

nfiles = n_elements(files)
;nmid = nfiles/2  now see below

fxo = intarr(nfiles)
fyo = intarr(nfiles)
fnx = intarr(nfiles)
fny = intarr(nfiles)
for f = 0, nfiles-1 do begin
   h = headfits(files[f])
   fxo[f] = -fix(sxpar(h, 'LTV1')) ; x origin in physical coordinate system
   fyo[f] = -fix(sxpar(h, 'LTV2')) ; y origin in physical coordinate system
   fnx[f] = sxpar(h, 'NAXIS1')
   fny[f] = sxpar(h, 'NAXIS2')
   ;print, sxpar(h,'PANE'), '    ', fnx[f], fny[f]
endfor

nx = min(fnx) ; could have an option to make these max, also.
ny = min(fny)
nmid = (where(fnx eq min(fnx) and fny eq min(fny))) [0]
xo = fxo[nmid]
yo = fyo[nmid]
hout = headfits(files[nmid])
;print, 'out: ', sxpar(hout,'PANE'), '    ', nx, ny


;refdata = mrdfits(files[nmid], 0, h, /silent, /fscale)
;s = size(refdata)
;nx = s[1]
;ny = s[2]

data = fltarr(nx, ny, nfiles) + !values.f_nan
inmeds = fltarr(nfiles)
usefiles = strarr(nfiles)

z = 0
for f = 0, nfiles-1 do begin
   if fxo[f] eq xo and fyo[f] eq yo and fnx[f] eq nx and fny[f] eq ny then begin
      indata = mrdfits(files[f], 0, h, /silent, /fscale)
      section = ''
   endif else begin
      readdata = mrdfits(files[f], 0, h, /silent, /fscale)
      indata = readdata[xo-fxo[f]:xo-fxo[f]+nx-1,yo-fyo[f]:yo-fyo[f]+ny-1] 
      section = '['+clip(xo-fxo[f])+'-'+clip(xo-fxo[f]+nx-1)+','+clip(yo-fyo[f])+'-'+clip(yo-fyo[f]+ny-1)+']'
   endelse

   ins = size(indata)
   inx = ins[1]
   iny = ins[2]
   if (inx ne nx) or (iny ne ny) then begin
      ; should no longer trigger.
      print, 'File ', files[f], ' has wrong dimensions ('+clip(inx)+'x'+clip(iny)+'; should have '+clip(nx)+'x'+clip(ny)+')'
      continue
   endif

   ;indata = indata + sxpar(h,'BZERO')
   if n_elements(normzonein) gt 0 then normzone = normzonein
   if n_elements(xnormkey) gt 0 then begin
     xnormstr = sxpar(h,xnormkey)
     xnormzone = fix(strsplit(xnormstr,',',/extract))
     normzone = [xnormzone[0], 0, xnormzone[1], -1]
   endif
   if n_elements(normzone) eq 4 then begin
      if normzone[2] lt 0 then normzone[2] = nx+normzone[2]
      if normzone[3] lt 0 then normzone[3] = ny+normzone[3]
      if normzone[0] lt 0 or normzone[1] lt 0 or normzone[2] ge nx or normzone[3] ge ny then print, 'WARNING - normzone extends outside image'
      inmed = sigclipmedian(indata[normzone[0]>0:normzone[2]<(nx-1),normzone[1]>0:normzone[3]<(ny-1)])
   endif else begin
      inmed = sigclipmedian(indata)
   endelse
   inmeds[f] = inmed
   if inmed ge mincounts and inmed le maxcounts then begin
      if keyword_set(indresp) then begin
         iresp = median(indata, dimension=2) ; if each flat has a different lamp spectrum (was heating up, etc.)...
         for x = 0, nx-1 do indata[x,*] = inmed * indata[x,*] / iresp[x] ; inmed will immediately get normalized out, which is what we want
      endif
      print, '  ' + removepath(files[f]) + section + ' (' + strtrim(long(inmed),2) + ' counts/pix)'
      data[*,*,z] = float(indata) / inmed
      usefiles[z] = files[f]
      z = z + 1
   endif else begin
      if inmed lt mincounts then $
      print, '  ' + removepath(files[f]) + section + ' (' + strtrim(long(inmed),2) + ' counts/pix) - too few counts; excluding'
      if inmed gt maxcounts then $
      print, '  ' + removepath(files[f]) + section + ' (' + strtrim(long(inmed),2) + ' counts/pix) - too many counts; excluding'
   endelse
endfor

if z lt 1 then begin ; formerly 3
    print, 'ERROR - Not enough counts to make a flat with these data!'
    return
endif

if z ne nfiles then data = data[*,*,0:z-1]

flat = fltarr(nx, ny)

if algorithm eq 'median' then begin
   print, '  Median-combining...'
   i = 0l
   for y = 0, ny-1 do begin    
     for x = 0, nx-1 do begin  ; x is the fast array and should be iterated inside
       i = i + 1
       flat[x, y] = median(data[x, y, *], /even) ; mediansigclip?
       if i mod 157557 eq 0 then begin
          st = stddev(data[x, y, *])
          val = flat[x, y]
          if keyword_set(verbose) then print, clip(x,5), clip(y,5), 'stdev=',clip(st,6), '  val=',clip(val,6);, '  stdev/sqrt(val)=',clip(st/sqrt(val),6) 
       endif
     endfor
   endfor
endif
if algorithm eq 'mean' then begin
   print,  '  Combining via trimmed mean...'
   i = 0l
   nslice = (size(data)) [3]
   for y = 0, ny-1 do begin    
     for x = 0, nx-1 do begin  ; x is the fast array and should be iterated inside
       slice = data[x,y,*]
       slice = slice[sort(slice)]
       slice = slice[trimlo:nslice-1-trimhi]
       flat[x,y] = mean(slice)
     endfor
   endfor
endif

if keyword_set(indresp) then begin
   ; Remove this again so the more refined response method can be applied later
   ; for x = 0, nx-1 do data[x,*,*] = data[x,*,*] * iresp[x]   ;  data not used again for spec
   for x = 0, nx-1 do flat[x,*]   = flat[x,*]   * iresp[x]   ; 
endif

;plot, flat[*,400]


if keyword_set(removeobjects) then begin
   print, '  Identifying objects...'

   if n_elements(objthresh) eq 0 then objthresh = 6
   if n_elements(objbuffer) eq 0 then objbuffer = 12
   for f = 0, z-1 do begin
      if keyword_set(verbose) then print, '  ', removepath(files[f])
      indata = data[*,*,f] / flat
      mask = fltarr(nx, ny)
      satmask = fltarr(nx, ny)

      sigclipstats, indata, sigmahi=5, sigmalo=5, median=datamed, stdev=datastdev
      ;datamed = sigclipmedian(indata, sigmahi = 5, sigmalo = 5)
      ;datastdev = sigclipstdev(indata, sigmahi = 5, sigmalo = 5)

      sourcepixels = where(indata ge datamed + objthresh * datastdev, ctsourcepix)
      if ctsourcepix gt 0 then begin
         mask[sourcepixels] = 1
         smoothmask = smooth(mask, 1+2*objbuffer, /edge_truncate)
      endif
      ctsatpix = 0
      if n_elements(satlevel) gt 0 then begin
         satpixels = where(indata ge satlevel, ctsatpix)
         if ctsatpix gt 0 then begin
            satmask[satpixels] = 1
            smoothsatmask = smooth(mask, 1+2*satradius, /edge_truncate)
         endif
      endif

      if ctsourcepix gt 0 then begin
         sourcepixels = where(smoothmask gt 1.0E-6, ct)
         indata[sourcepixels] = !Values.F_NAN
      endif
      if ctsatpix gt 0 then begin
         nearsatpixels = where(smoothsatmask gt 1.0E-6, ct)
         indata[nearsatpixels] = !Values.F_NAN
      endif


      data[*,*,f] = indata  ; inefficient to copy over unaffected elements, but oh well
   endfor


   reflat = fltarr(nx, ny)

   if algorithm eq 'median' then begin
     print, '  Median-combining...'
     i = 0l
     for y = 0, ny-1 do begin
       for x = 0, nx-1 do begin
         i = i + 1
         reflat[x, y] = median(data[x, y, *], /even)   ;sigclipping this is REALLY slow...
         if i mod 157557 eq 0 and keyword_set(verbose) then begin
            slice = data[x, y, *]
            good = where(finite(slice), ct)
            if ct gt 3 then st = stddev(slice[good]) else st = 0
            val = flat[x,y] * reflat[x, y]
            print, '  ', clip(x,5), clip(y,5), 'n=', clip(ct,2), ' stdev=',fpr(st,1.4), '  val=',fpr(val,1.4);, '  stdev/sqrt(val)=',fpr(st/sqrt(val),3.2) 
         endif
       endfor
     endfor
     bad = where(finite(reflat) eq 0, ct)
     if ct gt 0 then reflat[bad] = 1
   endif

   if algorithm eq 'mean' then begin
     print,  '  Combining via trimmed mean...'
     i = 0l
     for y = 0, ny-1 do begin
       for x = 0, nx-1 do begin
         slice = data[x,y,*]
         good = where(finite(slice) eq 1, ctgood)
         if ctgood eq 0 then begin
             reflat[x,y] = 1
         endif else begin
             slice = slice[good]
             itrimlo = trimlo
             itrimhi = trimhi
             while ctgood-itrimlo-itrimhi lt 1 do begin
               itrimlo = (itrimlo - 1) > 0
               itrimhi = (itrimhi - 1) > 0
             endwhile
             slice = slice[sort(slice)]
             slice = slice[itrimlo:ctgood-1-itrimhi]
             reflat[x,y] = mean(slice)
         endelse
       endfor
     endfor
   endif

   flat = flat * reflat    ; want to divide by flat, then divide by reflat, so divide by (flat * reflat)

endif



if keyword_set(response) then begin
  if n_elements(linewidth) eq 0 then linewidth = 12
  if n_elements(lineblocks) eq 0 then lineblocks = 5

  if n_elements(yresp) eq 0 then begin
    yresp = indgen(ny)
  endif
  if n_elements(lines) eq 55 then begin ; this is not used
    if lines eq 'find' then begin
        lines = [-1]
        nyr = n_elements(yresp)
        lineyresp = yresp[(nyr/2-20) > 0 : (nyr/2+20) < (nyr-1)]
        medcolx = fltarr(nx)
        for x = 0, nx-1 do begin
           medcolx[x] = median(flat[x,lineyresp], /even)
        endfor
        for x = 50, nx-50-1, 100 do begin
           xb     = (indgen(nx)) [(x-75)>0:(x+75)<(nx-1)]
           par = poly_fit(xb, medcolx[xb], 3, yfit=yfit)
           ydiff = medcolx[xb]-yfit
           posslines = where(ydiff gt 3.5*stddev(ydiff), ct) ; positive deviation only
           if ct gt 0 then print, xb[posslines]
           if ct gt 0 then lines = [lines,xb[posslines]]
        endfor
    endif
  endif

  medcolx = fltarr(nx)+!values.f_nan
  for x = 0, nx-1 do begin
     if total(finite(flat[x,yresp])) gt 0.75*ny then $
     medcolx[x] = median(flat[x,yresp], /even)
  endfor
  ;plot, medcolx
  
  ; could run a line detection algorithm here by doing a fit with a broad coverage
  ; and look for sharp deviances.

  dbound = [0, nx]
  if n_elements(xdiscontinuity) gt 0 then if xdiscontinuity[0] gt 0 then $
                                          dbound = [0, xdiscontinuity, nx]

  responsex = medcolx
  for x = 0, nx-1 do begin 
     if finite(medcolx[x]) eq 0 then continue
     d = max(where(x ge dbound))
     xi = (findgen(nx)) [(x-7)>dbound[d]:(x+7)<dbound[d+1]-1]  ; formerly +/- 10
     xi = xi[where(finite(medcolx[xi]))]
     ;par = polyiterfit(xi, medcolx[xi], order=2, yfit=yfit)
     par = poly_fit(xi-x, medcolx[xi], 3, yfit=yfit)
     ;delvarx, fpar
     ;for o = 1, 3 do begin
     ;  fpar = polyfitg(xi, medcolx[xi], o, yfit=yfit, guess=fpar)
     ;endfor
     ;if x eq 1660 then stop
     responsex[x] = yfit[where(xi eq x)]
  endfor
  for d = 1, n_elements(dbound)-2 do begin
     x = indgen(nx)
     responsex[where(x ge dbound[d])] += medcolx[dbound[d]-1] - medcolx[dbound[d]]
  endfor
  for x = 0, nx-1 do begin
     flat[x,*] = flat[x,*] / responsex[x]
  endfor

  ; mwrfits, flat, 'flatresp1.fits', /create

  ystart = fix(ny*(findgen(lineblocks)/lineblocks))
  ystop =  fix(ny*((1+findgen(lineblocks))/lineblocks))-1
  for l = 0, n_elements(lines)-1 do begin
     linestart = lines[l] - linewidth/2
     linestop = lines[l] + linewidth/2
     for b = 0, lineblocks-1 do begin
        medleft = median(flat[linestart-9:linestart-3,ystart[b]:ystop[b]], /even)
        medright = median(flat[linestop+3:linestop+9,ystart[b]:ystop[b]], /even)
        medtarget = (medleft + medright)/2.
        for x = linestart, linestop do begin
           medzone = median(flat[x,ystart[b]:ystop[b]], /even)
           flat[x,ystart[b]:ystop[b]] = medtarget * flat[x,ystart[b]:ystop[b]] / medzone
        endfor
     endfor
     medleft = median(flat[linestart-9:linestart-3,*], /even)
     medright = median(flat[linestop+3:linestop+9,*], /even)
     medtarget = (medleft + medright)/2.
     for x = linestart, linestop do begin
        flat[x,*] = medtarget * flat[x,*] / median(flat[x,*])
     endfor
  endfor

  ; don't even bother to flatfield for counts < ~1000*gain: even with 20 flatfields
  ; error introduced is 1% which exceeds typical flat variation

  reqcounts = 1000 < (max(medcolx)/10.)

  lowcounts = where(medcolx lt reqcounts, ct, complement=goodcounts) ; formerly medcolx*median(inmeds)
  maxx = (where(medcolx eq max(medcolx))) [0]
  if ct gt 0 then begin
    if min(lowcounts) lt maxx then $
       print, '  Low counts for x=', $
           clip(min(lowcounts[where(lowcounts lt maxx)])),'-',clip(max(lowcounts[where(lowcounts lt maxx)]))
    if max(lowcounts) gt maxx then $
       print, '  Low counts for x=', $
           clip(min(lowcounts[where(lowcounts gt maxx)])),'-',clip(max(lowcounts[where(lowcounts gt maxx)]))
    print, '  Using only a slit-averaged flat in this region.'
    for y = 0, ny-1 do begin
      flat[lowcounts,y] = median(flat[goodcounts[0:100<(n_elements(goodcounts)-1)],y])   
        ; this does assume directionality of the bad region...
    endfor
  endif

  ; in case there are unilluminated pixels, do a final overall division
  unilluminated = where(flat gt 0.5, ct)
  if ct gt 0 then begin
    med = median(flat[unilluminated], /even)
    flat /= med
  endif

endif

if keyword_set(slitnorm) then begin  ; halogen flats have an artificial slit response superimposed; remove it here.
  if n_elements(goodcounts) eq 0 then goodcounts = indgen(nx)
  for y= 0, ny-1 do begin
     medrowy = median(flat[goodcounts,y], /even)
     if medrowy gt 0.25 then flat[*,y] /= medrowy ; less than this is assumed to not be a false slit variation but genuinely 
                                                  ; dimmed part of the chip (off slit), which should be kept for later cropping
  endfor
  
endif


badval = where(flat le 0.01, ctbad)  ; zero values will create problems (nans) later and negative values are nonsense.
                                     ; also flagging very low values in advance.
if ctbad gt 0 then begin
  flat[badval] = 0.01  ; Prevents noise from being amplified too ridiculously if this flat is actually used.  Could set to Nan...
endif

sxaddpar, hout, 'BZERO', 0
for f = 0, z-1 do begin
  sxaddpar, hout, 'FLAT'+strtrim(string(f),2), removepath(usefiles[f])
endfor

if keyword_set(type) then sxaddpar,  hout, 'FLATTYPE', type

get_date, now
hist = 'Processed by flatcombine '+now
sxaddhist, hist, hout


sxaddpar, hout, 'BZERO', 0
sxaddpar, hout, 'BSCALE', 1
mwrfits, flat, outfile, hout, /create

print, '  Written to ', outfile

end
