function wmean, val, unc, meanunc=meanunc, sigclip=sigclip, nrej=nrej, good=good, rec=rec

  ; weighted mean based on uncertainties with 1/unc^2 weighting
  ; rec indicates recursion call (never set this at top level)

  weight =  1./unc^2  ; formerly was using 1/unc weighting!  which I guess is more robust...
  bad = where(weight eq 0 or finite(weight) eq 0, ctbad)
  if ctbad gt 0 then weight[bad] = 0.

  outmean = total(val * weight) / total(weight)

  relw = weight/total(weight)   

  meanunc = sqrt(total((relw*unc)^2.))

  scf = 1.0
  if keyword_set(rec) eq 0 then good = indgen(n_elements(val))
  if keyword_set(sigclip) then begin
     outmed = median([val[where(weight ge median([weight]))]], /even) ; robust estimator
     if sigclip lt 1.5 then print, 'Cannot sigma clip below 1.5'
     sigdev = abs(val-outmed)/unc
     outliers = where(sigdev gt sigclip, nrej, complement=ok)
     while nrej eq n_elements(sigdev) do begin
       scf *= 1.5
       outliers = where(sigdev gt sigclip*scf, nrej, complement=ok)
     endwhile
     good = good[ok]
     if nrej gt 0 then outmean = wmean(val[ok], unc[ok], meanunc=meanunc, good=good, rec=1)
  endif

  return, outmean

end

pro testwmean
   arr = randomn(seed,1000)
   unc = 1+fltarr(1000)
     arr[10] = 10
     arr[50] = 100
     arr[100] = 950
   window, 0
   i = findgen(1000)
   plot, i, arr, /nodata
   oplot, i, arr, psym=3, color=rgb(255,0,0)
   print, wmean(arr, unc, good=good)
   print, max(arr[good])
   print, wmean(arr, unc, sigclip=5, good=good)
   print, max(arr[good])
   oplot, i[good], arr[good], psym=3, color=rgb(0,255,0)
   help, good
stop
end
