; Rebin an LRIS image.  (could generalize...?)

; Use this if you took calibrations in 1x1 binning mode but switched to 2x2 (or 2x1, 1x2) mode later.

pro lrisrebin, imagename, newbinning, outimagename=outimagename, fluxsum=fluxsum

  if file_test(imagename) eq 0 then begin
     print, 'File not fount:', imagename
     return
  endif
  if n_elements(newbinning) eq 0 then begin
     print, "Specify new binning:  e.g. lrisrebin, 'image.fits', '2x2'"
     return
  endif
  if strpos(newbinning,',') gt 0 then splitchar = ','
  if strpos(newbinning,'x') gt 0 then splitchar = 'x'
  newbin = fix(strsplit(newbinning,splitchar,/extract))

  if n_elements(outimagename) eq 0 then outimagename = repstr(imagename,'.fits',repstr(newbinning,',','x')+'.fits')

  data = mrdfits(imagename,0,h, /silent)

  oldbinning = clip(sxpar(h,'BINNING'))
  oldbin = fix(strsplit(oldbinning,',',/extract))

  ; The CCD gap bug caused an offset that we needed to fix before.
  ; It is fixed at the prepare stage now so this code block is deprecated.
  ;if sxpar(h,'INSTRUME') eq 'LRISBLUE' and oldbin[1] eq 1 and newbin[1] eq 2 then $
  ;    data = shift(data, 0, 2)
  ;if sxpar(h,'INSTRUME') eq 'LRISBLUE' and oldbin[1] eq 2 and newbin[1] eq 1 then $
  ;    data = shift(data, 0, -oldbin[1])

  dim = (size(data))[1:*]

  binfactor = newbin*1.0/oldbin

  ; deal with binning down a data array that is sized by a number not divisible by the bin factor
  if binfactor[0] gt 1.0 then begin
    extrapix0 = dim[0] mod fix(dim[0]/binfactor[0])
    if extrapix0 gt 0 then data = data[0:dim[0]-1-extrapix0,*] ; perhaps better to expand?
  endif
  if binfactor[1] gt 1.0 then begin
    extrapix1 = dim[1] mod fix(dim[1]/binfactor[1] )
    if extrapix1 gt 0 then data = data[*,0:dim[1]-1-extrapix1]
  endif

  
  newdata = rebin(data, fix(dim[0]/binfactor[0]), fix(dim[1]/binfactor[1]), /sample)

  if keyword_set(fluxsum) then newdata *= binfactor[0]*binfactor[1]
  if keyword_set(fluxsum) then sxaddpar, h,'SATURATE',sxpar('SATURATE')*binfactor[1]*binfactor[2]

  newdim = (size(newdata)) [1:*]

  print,    imagename + ' ('+clip(   dim[0])+'x'+clip(   dim[1])+ ' @ '+repstr(oldbinning,',','x')+') --> ', $
         outimagename + ' ('+clip(newdim[0])+'x'+clip(newdim[1])+ ' @ '+repstr(newbinning,',','x')+')'

  sxaddpar, h, 'BINNING', repstr(newbinning,'x',','), 'Rebinned from'+oldbinning
  ; sxaddpar, h, 'XBIN', sxpar(h,'XBIN')*binfactor[0]
  ; sxaddpar, h, 'YBIN', sxpar(h,'YBIN')*binfactor[1]
  sxaddpar, h, 'LTV1', sxpar(h,'LTV1')/binfactor[0]
  sxaddpar, h, 'LTV2', sxpar(h,'LTV2')/binfactor[1]
  sxaddpar, h, 'LTM1_1', sxpar(h,'LTM1_1')/binfactor[0]
  sxaddpar, h, 'LTM2_1', sxpar(h,'LTM2_1')/binfactor[1]
  sxaddpar, h, 'LTM1_2', sxpar(h,'LTM1_2')/binfactor[0]
  sxaddpar, h, 'LTM2_2', sxpar(h,'LTM2_2')/binfactor[1]

  newl1 = strsplit(sxpar(h,'AMPL1'),',',/extract)/binfactor[1]
  newl2 = strsplit(sxpar(h,'AMPL2'),',',/extract)/binfactor[1]
  newr1 = strsplit(sxpar(h,'AMPR1'),',',/extract)/binfactor[1]
  newr2 = strsplit(sxpar(h,'AMPR2'),',',/extract)/binfactor[1]
  sxaddpar, h, 'AMPL1', clip(newl1[0])+','+clip(newl1[1])
  sxaddpar, h, 'AMPL2', clip(newl2[0])+','+clip(newl2[1])
  sxaddpar, h, 'AMPR1', clip(newr1[0])+','+clip(newr1[1])
  sxaddpar, h, 'AMPR2', clip(newr2[0])+','+clip(newr2[1])

  sxaddpar, h, 'REBIN', imagename, 'Original file before rebinning'
  disp = sxpar(h,'DISPERSN', count=ct)
  if ct gt 0 then sxaddpar, h, 'DISPERSN', disp*binfactor[0] ; assumes dispersion is in x-direction

  window = sxpar(h,'WINDOW', count=ct)
  if ct eq 0 then begin
    print, 'Warning: WINDOW keyword not found?'
    return
  endif else begin
    winstr = fix(strsplit(window, ',', /extract))
    newwindow = clip(winstr[0])+','+clip(winstr[1])+','+clip(winstr[2])+','+clip(fix(winstr[3]/binfactor[0]))+','+clip(fix(winstr[4]/binfactor[1]))
    sxaddpar, h, 'WINDOW', newwindow
  endelse

  ; leaving AMP?? alone for now

  mwrfits, newdata, outimagename, h, /create


end

